﻿init python:
    from copy import copy
    class mouse_follower(renpy.Displayable):
        def __init__(self, image = None, visible = False):
            super(renpy.Displayable,self).__init__()
            self.image = image
            self.visible = visible
            self.zoom = 3
        def change(self, image):
            self.image = image
            if image is None:
                self.visible = False
        def switch(self, stat):
            self.visible = stat
        def render(self,width,height,st,at):
            if self.visible and self.image:
                m = renpy.get_mouse_pos()
            else:
                m = (-300,-300)
            t = Transform(self.image, zoom = self.zoom, nearest = True)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (m[0],m[1]))

            renpy.redraw(self, 0)
            return rv

    class alchemy_object(renpy.Displayable):
        def __init__(self, image):
            super(renpy.Displayable,self).__init__()
            self.image = image
            self.level = 0
            self.zoom = 6.0
            self.zoom_add = 0
            self.rotation = 0
            self.rotation_direction = 1
            self.moving = 0
        def click(self):
            self.moving = 40
        def render(self,width,height,st,at):
            if self.moving:
                self.rotation += self.rotation_direction
                if self.rotation > 2:
                    self.rotation_direction = -1
                    self.zoom_add += .1
                elif self.rotation < -2:
                    self.rotation_direction = 1
                    self.zoom_add += -.1
                self.moving -= 1
                if self.moving < 0:
                    self.moving = 0
                    self.zoom_add = 0
            else:
                self.rotation = 0
            t = Transform(self.image, rotate = self.rotation, zoom = self.zoom + self.zoom_add, nearest = True)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (0, 0))

            renpy.redraw(self, 0)
            return rv
    class alchemy_bar(renpy.Displayable):
        def __init__(self):
            super(renpy.Displayable,self).__init__()
            self.image = "alchemy_bar_pointer"
            self.x = -350
            self.moving = True
            self.parent = None
            self.zoom = 6
            self.speed = 4
        def render(self,width,height,st,at):
            self.x += self.speed
            if self.x > 0:
                self.x = -350
                self.parent.sell()

            t = Transform(self.image, zoom = self.zoom, nearest = True)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (self.x, 0))

            renpy.redraw(self, 0)
            return rv
    class alchemy_potion:
        def __init__(self, name, info, value, image):
            self.name = name
            self.info = info
            self.value = value
            self.image = image
            self.level = 0
            self.size = 0
            self.quality = 0
        @property
        def get_image(self):
            return self.image + "_" + str(self.size)
        @property
        def get_price(self):
            return int(((self.level + self.size)*.1)*self.value) + self.value
    class alchemy_element(renpy.Displayable):
        def __init__(self, name, potions, image):
            super(renpy.Displayable,self).__init__()
            self.name = name
            self.potions = potions
            self.image = image
            self.level = 0
            self.zoom = 6.0
            self.zoom_add = 0
            self.rotation = 0
            self.rotation_direction = 1
            self.moving = 0
        def generate(self):
            potion =  copy(random.choice(self.potions))
            level = self.level + random.randint(-2,2)
            if level < 0:
                level = 0
            size = level
            if size > 8:
                size = 8
            size += random.randint(-3,3)
            if size < 0:
                size = 0
            elif size > 7:
                size = 7

            potion.size = size
            potion.level = level
            return potion
        def click(self):
            self.moving = 40
        def render(self,width,height,st,at):
            if self.moving:
                self.rotation += self.rotation_direction
                if self.rotation > 2:
                    self.rotation_direction = -1
                    self.zoom_add += .1
                elif self.rotation < -2:
                    self.rotation_direction = 1
                    self.zoom_add += -.1
                self.moving -= 1
                if self.moving < 0:
                    self.moving = 0
                    self.zoom_add = 0
            else:
                self.rotation = 0
            t = Transform(self.image, rotate_pad = False, rotate = self.rotation, zoom = self.zoom, nearest = True)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (0, 0))

            renpy.redraw(self, 0)
            return rv


    class alchemy_class:
        def __init__(self, elements, sell_bar, brew_bar):
            self.cash = 0
            self.elements = elements
            self.potions = []
            self.collection = []
            self.cauldron = alchemy_object("alchemy_cauldron")
            self.mouse_follower = mouse_follower("alchemy_cauldron")
            self.economy = None
            self.faeries = []

            self.current_element = None
            self.sell_bar = sell_bar
            self.brew_bar = brew_bar
            sell_bar.parent = self
            brew_bar.parent = self
        def brew(self):
            self.cauldron.click()
            if self.current_element == None:
                element = copy(random.choice(self.elements))
            else:
                element = self.current_element
            renpy.play("alchemy/brew.ogg", "sound")
            potion = element.generate()
            self.potions.append(potion)
            tag = "ap" + str(random.randint(1, 10000))
            Show("alchemy_potion_pop", img = potion.get_image, tag = tag, _tag = tag)()
        def sell(self):
            if len(self.potions):
                potion = self.potions.pop(0)
                cash = potion.get_price
                self.cash += cash
                floating_text(cash, 1805, 150)
                renpy.restart_interaction()
                renpy.play("alchemy/coin.ogg", "sfx1")
        def sell_bulk(self):
            if len(self.potions) < 1:
                return
            total = 0
            for i in self.potions:
                total += i.get_price
            self.cash += total//2
            floating_text(total//2)
            self.potions = []
            self.sell_bar.x = -350
            renpy.play("alchemy/ka-ching.ogg", "sound")
        def collect(self, potion):
            self.potions.remove(potion)
            self.collection.append(potion)
        def set_action(self, action):
            self.action == action
        def select(self, obj):
            self.current_element = obj
            if obj is None:
                self.mouse_follower.change(None)
                self.mouse_follower.switch(False)
            else:
                self.mouse_follower.change(obj.image)
                self.mouse_follower.switch(True)
        def upgrade(self, obj):
            if self.cash >= 1000:
                self.cash -= 1000
                obj.level += 1
        def sweep(self, obj):
            cash = obj.level*600
            self.cash += cash
            floating_text(cash)
            obj.level = 0
        def swap(self, obj):
            pass
        def clicked(self, target):
            target.click()
            m = renpy.get_mouse_pos()
            Show("alchemy_element_selected", g = self, obj = target, x = m[0], y = m[1])()

        def test(self, n):
            if n == 1:
                self.cash += 1000


init:
    default container_sizes = [
        "Tube",
        "Vial",
        "Shot",
        "Small flask",
        "Large flask",
        "Quart",
        "Bottle",
        "Pint",
        ]
    image alchemy_frame = Crop((112,16,16,16), "alchemy/sprite.png")
    image alchemy_frame_6x:
        zoom(6)
        nearest True
        Crop((112,16,16,16), "alchemy/sprite.png")
    image alchemy_bar_pointer = Crop((112,0,7,8), "alchemy/sprite.png")
    image alchemy_cauldron = Crop((0,0,80,64), "alchemy/sprite.png")
    image alchemy_fire_pit:
        Crop((80,0,32,16), "alchemy/sprite.png")
        .1
        Crop((80,16,32,16), "alchemy/sprite.png")
        .1
        Crop((80,32,32,16), "alchemy/sprite.png")
        .1
        Crop((80,48,32,16), "alchemy/sprite.png")
        .1
        repeat
    image alchemy_coin:
        zoom(6)
        nearest True
        Crop((113,49,5,6), "alchemy/sprite.png")
        .1
        Crop((118,49,5,6), "alchemy/sprite.png")
        .1
        Crop((123,49,5,6), "alchemy/sprite.png")
        .1
        Crop((118,49,5,6), "alchemy/sprite.png")
        .1
        repeat
    


    image alchemy_fire = Crop((0,64,32,32), "alchemy/sprite.png")
    image health_potion_0 = Crop((32,64,16,32), "alchemy/sprite.png")
    image health_potion_1 = Crop((48,64,16,32), "alchemy/sprite.png")
    image health_potion_2 = Crop((64,64,16,32), "alchemy/sprite.png")
    image health_potion_3 = Crop((80,64,16,32), "alchemy/sprite.png")
    image health_potion_4 = Crop((96,64,16,32), "alchemy/sprite.png")
    image health_potion_5 = Crop((112,64,16,32), "alchemy/sprite.png")
    image health_potion_6 = Crop((128,64,16,32), "alchemy/sprite.png")
    image health_potion_7 = Crop((144,64,16,32), "alchemy/sprite.png")

    default health_potion = alchemy_potion("Health potion", "Invigorates.", 10, "health_potion")
    default alchemy_fire = alchemy_element(
        "Fire",
        [health_potion],
        "alchemy_fire"
        )

    image alchemy_water = Crop((0,96,32,32), "alchemy/sprite.png")
    image mana_potion_0 = Crop((32,96,16,32), "alchemy/sprite.png")
    image mana_potion_1 = Crop((48,96,16,32), "alchemy/sprite.png")
    image mana_potion_2 = Crop((64,96,16,32), "alchemy/sprite.png")
    image mana_potion_3 = Crop((80,96,16,32), "alchemy/sprite.png")
    image mana_potion_4 = Crop((96,96,16,32), "alchemy/sprite.png")
    image mana_potion_5 = Crop((112,96,16,32), "alchemy/sprite.png")
    image mana_potion_6 = Crop((128,96,16,32), "alchemy/sprite.png")
    image mana_potion_7 = Crop((144,96,16,32), "alchemy/sprite.png")

    default mana_potion = alchemy_potion("Mana potion", "A flow of mana.", 10, "mana_potion")
    default alchemy_water = alchemy_element(
        "Water",
        [mana_potion],
        "alchemy_water"
        )

    image alchemy_earth = Crop((0,128,32,32), "alchemy/sprite.png")
    image strength_potion_0 = Crop((32,128,16,32), "alchemy/sprite.png")
    image strength_potion_1 = Crop((48,128,16,32), "alchemy/sprite.png")
    image strength_potion_2 = Crop((64,128,16,32), "alchemy/sprite.png")
    image strength_potion_3 = Crop((80,128,16,32), "alchemy/sprite.png")
    image strength_potion_4 = Crop((96,128,16,32), "alchemy/sprite.png")
    image strength_potion_5 = Crop((112,128,16,32), "alchemy/sprite.png")
    image strength_potion_6 = Crop((128,128,16,32), "alchemy/sprite.png")
    image strength_potion_7 = Crop((144,128,16,32), "alchemy/sprite.png")

    default strength_potion = alchemy_potion("Strength potion", "Strong as rock..", 10, "strength_potion")
    default alchemy_earth = alchemy_element(
        "Earth",
        [strength_potion],
        "alchemy_earth"
        )

    image alchemy_air = Crop((0,160,32,32), "alchemy/sprite.png")
    image agility_potion_0 = Crop((32,160,16,32), "alchemy/sprite.png")
    image agility_potion_1 = Crop((48,160,16,32), "alchemy/sprite.png")
    image agility_potion_2 = Crop((64,160,16,32), "alchemy/sprite.png")
    image agility_potion_3 = Crop((80,160,16,32), "alchemy/sprite.png")
    image agility_potion_4 = Crop((96,160,16,32), "alchemy/sprite.png")
    image agility_potion_5 = Crop((112,160,16,32), "alchemy/sprite.png")
    image agility_potion_6 = Crop((128,160,16,32), "alchemy/sprite.png")
    image agility_potion_7 = Crop((144,160,16,32), "alchemy/sprite.png")

    default agility_potion = alchemy_potion("Agility potion", "Move like the wind.", 10, "agility_potion")
    default alchemy_air = alchemy_element(
        "Air",
        [agility_potion],
        "alchemy_air"
        )

    # default alchemy_experiment = alchemy_element("Experiment", [])
    # default alchemy_soul = alchemy_element("Soul", [])



style alchemy2_frame:
    background Frame("alchemy_frame_6x", 10,10)
    padding (20,20)
style alchemy2_button:
    background Frame("alchemy_frame_6x", 10,10)
    padding (20,20)
style alchemy2_text:
    font "fonts/chary___.ttf"
    size 58
style alchemy_text:
    font "fonts/chary___.ttf"
    size 58
screen alchemy_element_selected(g, obj, x, y):
    modal True
    style_prefix "alchemy2"
    button:
        background None
        action Hide("alchemy_element_selected")
    drag:
        pos x,y
        frame:
            vbox:
                spacing 8
                text "Level [obj.level] [obj.name] element."
                hbox:
                    spacing 8
                    button:
                        at alchemy_highlight
                        text "Select"
                        action Function(g.select, obj), Hide("alchemy_element_selected")
                    button:
                        at alchemy_highlight
                        insensitive_background "#0004"
                        text "Upgrade" insensitive_color "#555"
                        action Function(g.upgrade, obj), SensitiveIf(g.cash > 999)
                hbox:
                    spacing 8
                    text "Next upgrade: 1000" yalign .5
                    add "alchemy_coin" yalign .5 
                hbox:
                    spacing 8
                    button:
                        at alchemy_highlight
                        text "Swap"
                        action Function(g.swap, obj), Hide("alchemy_element_selected")
                    button:
                        at alchemy_highlight
                        text "Sweep"
                        action Function(g.sweep, obj), Hide("alchemy_element_selected")

transform alchemy_potion_pop_animation(tag):
    nearest True
    zoom 6
    ease .2 yoffset -100
    parallel:
        ease .2 yoffset 100
    parallel:
        ease .5 xoffset 1000 alpha 0

screen alchemy_potion_pop(img, tag):
    add img align .5,.3 at alchemy_potion_pop_animation(tag)
    timer .5 action Hide(tag)
screen alchemy_screen():
    modal True
    style_prefix "alchemy"
    default g = alchemy_class(
        [alchemy_fire, alchemy_water, alchemy_earth, alchemy_air],
        alchemy_bar(),
        alchemy_bar(),
        )
    fixed fit_first True:
        at zoom(6, True)
        add "alchemy_background"
        add "alchemy_fire_pit" align .5,.0 yoffset 98
    button:
        align .5,.5 yoffset -30 padding 0,0
        add g.cauldron
        action Function(g.brew)
    vbox:
        align .0,.0 spacing -24 offset 18,126
        for n,i in enumerate(g.elements):
            button:
                xysize 192, 192
                at alchemy_highlight
                if n % 2:
                    xoffset 144
                padding 0,0
                add i
                text str(i.level) align 1.0,1.0 outlines[(6,"#291d24",0,0)] yoffset 12
                action Function(g.clicked, i)
    vbox:
        align 1.0,.0 spacing -24 offset -18,126
        for n,i in zip([0,1,2,3], g.potions):
            button:
                xysize 192, 192
                at alchemy_highlight
                if n % 2:
                    xoffset -144
                padding 0,0 selected_background Frame("alchemy_frame", 10,10)
                fixed fit_first True align .5,.5:
                    add i.get_image align .5,.5 at zoom(6,True)
                    text str(i.level) align 1.0,1.0 outlines[(6,"#291d24",0,0)] yoffset 12
                action Function(g.collect, i)
    if len(g.potions):
        add g.sell_bar align 1.0,.05
    hbox:
        align .5,.9 spacing 10
        text str(g.cash) yalign .5
        add "alchemy_coin" yalign .5
    vbox:
        align 1.0,.86
        text "{}".format(len(g.potions)) align 1.0,.5 xoffset -10
        button:
            background Frame("alchemy_frame_6x", 10,10)
            padding (20,20)
            text "Sell bulk"
            action Function(g.sell_bulk)
    vbox:
        align 0.0,.86
        text " " align .5,.5
        button:
            background Frame("alchemy_frame_6x", 10,10)
            padding (20,20)
            text "Select None"
            action Function(g.select, None)

    add g.mouse_follower
    # vbox:
    #     button:
    #         text "Add cash"
    #         action Function(g.test, 1)

transform alchemy_highlight:
    on idle:
        ease .2 additive 0.0 alpha 1.0
    on hover:
        ease .2 additive 0.1 alpha 1.0
    on insensitive:
        ease .2 additive 0.0 alpha .1

label alchemy_example:
    call screen alchemy_screen